package com.wrongcoder.ld48.lowmemory.core;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Preferences;
import com.badlogic.gdx.utils.Logger;

/** Registry and initializer of global resources */
public class Registry {

	public static final String GAME_NAME = "Low Memory";
	public static final String PREF_TAG = "low-memory";

	public static final int WIDTH = 800;
	public static final int HEIGHT = 600;

	public final Game game;
	public final Logger log;
	public final AssetManager assetManager;
	public final ScoreTransmitter scoreTransmitter;

	public final String platformId;
	public final Runnable platformInitializer;
	private boolean initialized = false;

	public Registry(final String platformId) {
		this(platformId, null);
	}

	public Registry(final String platformId, final Runnable platformInitializer) {
		// This happens before Gdx initialization
		this.game = new Game(this);
		this.log = new Logger(GAME_NAME, Logger.DEBUG);
		this.assetManager = new AssetManager();
		this.scoreTransmitter = new ScoreTransmitter(this);
		this.platformId = platformId;
		this.platformInitializer = platformInitializer;
	}

	/** Call exactly once in {@link Game#create()} */
	public void initialize() {
		if (initialized) {
			throw new IllegalStateException();
		}
		initialized = true;

		// This happens after Gdx initialization

		assetManager.queueAssets();

		if (platformInitializer != null) {
			platformInitializer.run();
		}
	}

	public String getClientId() {
		final Preferences preferences = Gdx.app.getPreferences(PREF_TAG);

		String clientId = preferences.getString("clientId", "");
		if (clientId.isEmpty()) {
			clientId = generateId();
			preferences.putString("clientId", clientId);
			preferences.flush();
		}
		return clientId;
	}

	public void setGameWinFlag() {
		final Preferences preferences = Gdx.app.getPreferences(PREF_TAG);
		preferences.putBoolean("completed", true);
		preferences.flush();
	}

	public boolean getGameWinFlag() {
		final Preferences preferences = Gdx.app.getPreferences(PREF_TAG);
		return preferences.getBoolean("completed", false);
	}

	public static String generateId() {
		//noinspection SpellCheckingInspection
		final String chars = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";
		final int numChars = chars.length();

		final StringBuilder sb = new StringBuilder(16);
		for (int i = 0; i < 16; i++) {
			final int c = (int) Math.floor(Math.random() * numChars);
			sb.append(chars.charAt(c));
		}

		return sb.toString();
	}

}
